from jija.config.base import Base

from pathlib import Path as Path
import sys


class StructureConfig(Base):
    PROJECT_PATH = None
    CORE_PATH = None
    APPS_PATH = None
    PYTHON_PATH = None

    def __init__(self, *, project_path=None, core_dir='core', apps_dir='apps', python_path=None, **kwargs):
        StructureConfig.PROJECT_PATH = self.__get_project_path(project_path)
        StructureConfig.CORE_PATH = StructureConfig.PROJECT_PATH.joinpath(core_dir)
        StructureConfig.APPS_PATH = StructureConfig.PROJECT_PATH.joinpath(apps_dir)
        StructureConfig.PYTHON_PATH = python_path or sys.executable

        super().__init__(**kwargs)

    @staticmethod
    def __get_project_path(project_path):
        if isinstance(project_path, Path):
            if not project_path.is_absolute():
                return project_path.absolute()

            return project_path

        path = Path(project_path) if project_path else Path()
        return path.absolute()



from jija.config.base import Base


class ProjectConfig(Base):
    SECRET_KEY = None

    def __init__(self, *, secret_key=b'*' * 32, **kwargs):
        ProjectConfig.SECRET_KEY = secret_key
        super().__init__(**kwargs)



import jija.config
from jija.config.base import Base


class DatabaseConfig(Base):
    REQUIRED = False

    DATABASE = None
    PASSWORD = None
    USER = None
    PORT = None
    HOST = None

    def __init__(self, *, database, password, host='localhost', user='postgres', port=5432, **kwargs):
        DatabaseConfig.DATABASE = database
        DatabaseConfig.PASSWORD = password
        DatabaseConfig.USER = user
        DatabaseConfig.PORT = port
        DatabaseConfig.HOST = host

        super().__init__(**kwargs)

    @classmethod
    async def load(cls):
        from jija_orm import config

        await config.JijaORM.async_init(
            project_dir=jija.config.StructureConfig.PROJECT_PATH,
            connection=config.Connection(
                host=cls.HOST,
                port=cls.PORT,
                user=cls.USER,
                database=cls.DATABASE,
                password=cls.PASSWORD
            ),
            apps=await cls.__get_apps()
        )

    @classmethod
    async def __get_apps(cls):
        from jija_orm import config
        from jija.apps import Apps

        apps = []
        for app in Apps.apps.values():
            if app.exist('models.py'):
                apps.append(config.App(name=app.name, migration_dir=app.get_import_path('migrations')))

        return apps



from jija.config.base import Base


class NetworkConfig(Base):
    HOST = None
    PORT = None

    def __init__(self, *, host='0.0.0.0', port=8080, **kwargs):
        NetworkConfig.HOST = host
        NetworkConfig.PORT = port

        super().__init__(**kwargs)
